"""The CI/CD stage - implements jobs to validate code quality.
"""

import aws_cdk as cdk

from constructs import Construct

from cdk_opinionated_constructs.stacks.code_quality_stack import CodeQualityStack


class CodeQualityStage(cdk.Stage):
    """The CodeQualityStage class implements the CI/CD code quality validation
    stage.

    Parameters:

    - scope (Construct): The parent construct that this stage will be added to.
    - construct_id (str): The id of this construct.
    - env (cdk.Environment): The CDK environment.
    - props (dict): Properties passed to the CodeQualityStack.
    - **kwargs: Additional keyword arguments passed to the base Stage constructor.

    The stage contains a single CodeQualityStack which implements the code quality jobs.
    """

    def __init__(self, scope: Construct, construct_id: str, env: cdk.Environment, props: dict, **kwargs) -> None:
        super().__init__(scope, construct_id, env=env, **kwargs)

        CodeQualityStack(
            self,
            construct_id="codequality-stack",
            env=env,
            props=props,
        )



"""Opinionated CDK construct to create an SNS topic.

Security parameters are set by default
"""

import aws_cdk.aws_sns as sns

from aws_cdk import aws_iam as iam, aws_kms as kms
from constructs import Construct


class SNSTopic(Construct):
    """CDK SNS topic construct."""

    def __init__(self, scope: Construct, id: str):  # noqa: A002
        super().__init__(scope, id)

    def create_sns_topic(self, topic_name: str, master_key: kms.IKey | None) -> sns.Topic | sns.ITopic:
        """Creates an SNS topic with opinionated settings.

        Parameters:

        - topic_name: Name of the SNS topic to create.

        - master_key: Optional KMS key to encrypt the topic.

        Returns: The created SNS topic object.

        It creates the SNS topic with the given name.

        It sets a resource policy to enforce SSL for publication requests.

        This prevents unencrypted publishing over HTTP.
        """

        topic = sns.Topic(self, id=topic_name, topic_name=topic_name, master_key=master_key)
        topic.add_to_resource_policy(
            iam.PolicyStatement(
                sid="AllowPublishThroughSSLOnly",
                actions=["sns:Publish"],
                effect=iam.Effect.DENY,
                resources=[topic.topic_arn],
                conditions={
                    "Bool": {"aws:SecureTransport": "false"},
                },
                principals=[iam.AnyPrincipal()],
            )
        )
        return topic



"""Helper functions to make your life simple."""

from pathlib import Path

import aws_cdk as cdk

from aws_cdk import Stack, Stage


def check_ansible_dir(directory: str) -> bool:
    """Checks if an Ansible directory exists.

    Parameters:
      - directory (str): The directory path to check.

    Returns:
      - bool: True if the directory exists, False otherwise.

    Functionality:
      - Resolves the full path using the __file__ variable.
      - Checks if the path exists and is a directory using pathlib.
      - Returns True if it's a directory, False otherwise.
    """

    this_dir = Path(__file__).parent
    ansible_path = this_dir.joinpath(directory).resolve()
    return ansible_path.is_dir()


def apply_tags(props: dict, resource: cdk.Stack | cdk.Stage) -> Stack | Stage:
    """Applies tags to a CDK Stack or Stage.

    Parameters:
      - props (dict): Dictionary containing tags to apply
      - resource (cdk.Stack | cdk.Stage): Stack or Stage to apply tags to

    Returns:
      - Stack | Stage: The same resource passed in, after tags applied

    Functionality:
      - Loops through tags in props and adds them to the resource with cdk.Tags.of
      - Returns the modified resource object
    """

    for key, value in props["tags"].items():
        cdk.Tags.of(resource).add(key, value)

    return resource



"""Opinionated CDK construct to create Network load balancer.

Security parameters are set by default
"""

import aws_cdk as cdk
import aws_cdk.aws_certificatemanager as certificate_manager
import aws_cdk.aws_elasticloadbalancingv2 as albv2
import aws_cdk.aws_events_targets as albv2_targets
import aws_cdk.aws_iam as iam
import aws_cdk.aws_s3 as s3

from cdk_nag import NagPackSuppression, NagSuppressions
from constructs import Construct

from cdk_opinionated_constructs.s3 import S3Bucket


class NetworkLoadBalancer(Construct):
    """Create Network LB."""

    def __init__(self, scope: Construct, construct_id: str):
        super().__init__(scope, construct_id)

    def create_access_logs_bucket(self, bucket_name: str, expiration_days: int) -> s3.Bucket | s3.IBucket:
        """Creates an S3 bucket for ALB access logs with required permissions.

        Parameters:

        - bucket_name: Name of the S3 bucket.
        - expiration_days: Number of days before objects expire.

        Returns:
            The S3 Bucket object.

        It creates the bucket with S3 managed encryption enabled.

        It adds a bucket policy to allow delivery.logs.amazonaws.com service principal to write logs.

        It allows the service principal to get bucket ACL.

        It adds a lifecycle rule to expire objects after the given days.

        It suppresses some false positive alerts from cfn-nag for this valid use case.
        """

        alb_access_logs_bucket_construct = S3Bucket(self, id=f"alb_access_logs_{bucket_name}_construct")
        alb_access_logs_bucket = alb_access_logs_bucket_construct.create_bucket(
            bucket_name=bucket_name, encryption=s3.BucketEncryption.S3_MANAGED
        )

        alb_access_logs_bucket.add_to_resource_policy(
            permission=iam.PolicyStatement(
                effect=iam.Effect.ALLOW,
                actions=["s3:PutObject"],
                principals=[iam.ServicePrincipal(service="delivery.logs.amazonaws.com")],
                resources=[f"{alb_access_logs_bucket.bucket_arn}/*"],
                conditions={"StringEquals": {"s3:x-amz-acl": "bucket-owner-full-control"}},
            )
        )

        alb_access_logs_bucket.add_to_resource_policy(
            permission=iam.PolicyStatement(
                effect=iam.Effect.ALLOW,
                actions=["s3:GetBucketAcl"],
                principals=[iam.ServicePrincipal(service="delivery.logs.amazonaws.com")],
                resources=[alb_access_logs_bucket.bucket_arn],
            )
        )
        alb_access_logs_bucket.add_lifecycle_rule(expiration=cdk.Duration.days(expiration_days))

        # Supress a few false positive alerts from the cfn-nag
        NagSuppressions.add_resource_suppressions(
            alb_access_logs_bucket,
            [
                NagPackSuppression(
                    id="AwsSolutions-S1",
                    reason="ALB access logs location, doesn't contain sensitive data it doesn't require "
                    "another resource for storing access logs from it",
                ),
            ],
        )

        return alb_access_logs_bucket

    @staticmethod
    def add_connections(
        nlb: albv2.NetworkLoadBalancer, certificates: list[certificate_manager.ICertificate], ports: list
    ):
        """Adds listeners and target groups to a Network Load Balancer.

        Parameters:

        - nlb: The NLB object to add listeners and targets to.

        - certificates: List of ACM certificates to add to listeners.

        - ports: List of port definitions, each containing:

          - front_end_protocol: Protocol for listener (TCP, TLS etc.)
          - front_end_port: Listener port number
          - back_end_protocol: Protocol for targets (TCP, HTTP etc.)
          - back_end_port: Target group port
          - targets: List of target IP addresses or ALBs
          - target_type: 'instance' or 'alb'
          - health_check_path: Path for health checks (default '/')
          - stickiness: Whether to enable stickiness (true/false)

        It creates a listener with provided certificates if given.

        It adds a target group for each port definition.

        It enables connection draining and custom health checks.

        It enables stickiness if configured.

        Example usage:
        add_connections(
            nlb=imported_network_load_balancer,
            certificates=[],
            ports=[
                {
                    "front_end_port": 6001,
                    "front_end_protocol": albv2.Protocol.UDP,
                    "targets": [service],
                    "back_end_port": 6001,
                    "back_end_protocol": albv2.Protocol.UDP,
                },
            ])
        """

        for port_definition in ports:
            front_end_protocol: albv2.Protocol = port_definition["front_end_protocol"]
            front_end_port: int = port_definition["front_end_port"]

            if certificates:
                listener = nlb.add_listener(
                    id=f"{front_end_protocol}-{front_end_port}",
                    port=front_end_port,
                    protocol=front_end_protocol,
                    certificates=certificates,
                    ssl_policy=port_definition.get("ssl_policy"),
                )

            else:
                listener = nlb.add_listener(
                    id=f"{front_end_protocol}-{front_end_port}", port=front_end_port, protocol=front_end_protocol
                )

            back_end_protocol: albv2.Protocol = port_definition["back_end_protocol"]
            back_end_port: int = port_definition["back_end_port"]
            targets: list[albv2_targets] = port_definition["targets"]
            if port_definition.get("target_type") == "alb":
                target = listener.add_targets(
                    id=f"{back_end_protocol}-{back_end_port}",
                    targets=targets,
                    port=back_end_port,
                    preserve_client_ip=True,
                    health_check=albv2.HealthCheck(
                        enabled=True,
                        protocol=albv2.Protocol.HTTPS,
                        port=str(back_end_port),
                        path=port_definition.get("health_check_path", "/"),
                    ),
                )
            else:
                target = listener.add_targets(
                    id=f"{back_end_protocol}-{back_end_port}",
                    targets=targets,
                    port=back_end_port,
                    preserve_client_ip=True,
                )
                target.set_attribute("deregistration_delay.connection_termination.enabled", "true")

            if port_definition.get("stickiness"):
                target.set_attribute("stickiness.enabled", "true")



import aws_cdk as cdk

from constructs import Construct

from cdk_opinionated_constructs.stacks.infrastructure_tests_stack import InfrastructureTestsStack


class InfrastructureTestsStage(cdk.Stage):
    """InfrastructureTestsStage defines a CDK Stage for infrastructure testing.

    It creates an InfrastructureTestsStack, passing along the stage props.

    Parameters:

    - scope: The CDK scope constructing this stage.
    - construct_id: ID for the stage construct.
    - env: The CDK environment.
    - props: Configuration properties passed to the stage.
    - **kwargs: Additional stage options.
    """

    def __init__(self, scope: Construct, construct_id: str, env: cdk.Environment, props: dict, **kwargs) -> None:
        super().__init__(scope, construct_id, env=env, **kwargs)

        InfrastructureTestsStack(
            self,
            construct_id="infrastructure-tests-stack",
            env=env,
            props=props,
        )



"""Opinionated CDK construct to create AWS WAFv2.

Security parameters, logging are set by default
"""

from typing import Any, Literal

import aws_cdk as cdk
import aws_cdk.aws_wafv2 as wafv2

from aws_cdk import aws_logs as logs
from constructs import Construct


class WAFv2(Construct):
    """Implement a v2 WAF where logs are sent to the AWS CloudWatch logs."""

    def __init__(self, scope: Construct, construct_id: str) -> None:
        super().__init__(scope, construct_id)

    @staticmethod
    def __aws_account_takeover_prevention(aws_account_takeover_prevention):
        """Creates a WAF rule to enable AWS account takeover prevention.

        Parameters:

        - aws_account_takeover_prevention: Dictionary containing:

          - login_path: The login page path
          - password_field: Password field identifier
          - username_field: Username field identifier

        It creates a rule to use the AWSManagedRulesATPRuleSet managed rule set.

        This is configured with the provided login path, password, and username fields.

        It enables CloudWatch metrics and sampled request logging.

        Returns the rule property to add to the WAF WebACL.
        """

        return wafv2.CfnWebACL.RuleProperty(
            name="AWS-AWSManagedRulesATPRuleSet",
            priority=6,
            override_action=wafv2.CfnWebACL.OverrideActionProperty(none={}),
            statement=wafv2.CfnWebACL.StatementProperty(
                managed_rule_group_statement=wafv2.CfnWebACL.ManagedRuleGroupStatementProperty(
                    name="AWSManagedRulesATPRuleSet",
                    vendor_name="AWS",
                    managed_rule_group_configs=[
                        wafv2.CfnWebACL.ManagedRuleGroupConfigProperty(
                            login_path=aws_account_takeover_prevention["login_path"],
                        ),
                        wafv2.CfnWebACL.ManagedRuleGroupConfigProperty(
                            password_field=wafv2.CfnWebACL.FieldIdentifierProperty(
                                identifier=aws_account_takeover_prevention["password_field"]
                            ),
                        ),
                        wafv2.CfnWebACL.ManagedRuleGroupConfigProperty(
                            payload_type="FORM_ENCODED",
                        ),
                        wafv2.CfnWebACL.ManagedRuleGroupConfigProperty(
                            username_field=wafv2.CfnWebACL.FieldIdentifierProperty(
                                identifier=aws_account_takeover_prevention["username_field"]
                            ),
                        ),
                    ],
                )
            ),
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True,
                metric_name="AWS-AWSManagedRulesATPRuleSet",
                sampled_requests_enabled=True,
            ),
        )

    @staticmethod
    def __aws_sqli_rule():
        """Creates a WAF rule to enable AWS managed SQL injection protection.

        It creates a rule to use the AWSManagedRulesSQLiRuleSet managed
        rule set.

        This enables protection against SQL injection attacks.

        It enables CloudWatch metrics and sampled request logging.

        Returns the rule property to add to the WAF WebACL.
        """

        return wafv2.CfnWebACL.RuleProperty(
            name="AWS-AWSManagedRulesSQLiRuleSet",
            priority=5,
            override_action=wafv2.CfnWebACL.OverrideActionProperty(none={}),
            statement=wafv2.CfnWebACL.StatementProperty(
                managed_rule_group_statement=wafv2.CfnWebACL.ManagedRuleGroupStatementProperty(
                    name="AWSManagedRulesSQLiRuleSet",
                    vendor_name="AWS",
                )
            ),
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True,
                metric_name="AWS-AWSManagedRulesSQLiRuleSet",
                sampled_requests_enabled=True,
            ),
        )

    @staticmethod
    def __aws_bad_inputs_rule():
        """Creates a WAF rule to block known bad inputs.

        It creates a rule using the AWSManagedRulesKnownBadInputsRuleSet
        managed rule set.

        This protects against input-based attacks like XSS, SQLi, etc.

        It enables CloudWatch metrics and sampled request logging.

        Returns the rule property to add to the WAF WebACL.
        """

        return wafv2.CfnWebACL.RuleProperty(
            name="AWS-AWSManagedRulesKnownBadInputsRuleSet",
            priority=4,
            override_action=wafv2.CfnWebACL.OverrideActionProperty(none={}),
            statement=wafv2.CfnWebACL.StatementProperty(
                managed_rule_group_statement=wafv2.CfnWebACL.ManagedRuleGroupStatementProperty(
                    name="AWSManagedRulesKnownBadInputsRuleSet",
                    vendor_name="AWS",
                )
            ),
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True,
                metric_name="AWS-AWSManagedRulesKnownBadInputsRuleSet",
                sampled_requests_enabled=True,
            ),
        )

    @staticmethod
    def __rate_list(rate_value: int):
        """Creates a WAF rate-based rule to limit requests per IP.

        Parameters:

        - rate_value: The maximum requests per 5 minutes per IP.

        It creates a rule to limit requests per IP based on the rate value.

        Requests exceeding the limit will be blocked.

        It enables CloudWatch metrics and sampled request logging.

        Returns the rule property to add to the WAF WebACL.
        """

        return wafv2.CfnWebACL.RuleProperty(
            name=f"Custom-RateLimit{rate_value}",
            priority=1,
            action=wafv2.CfnWebACL.RuleActionProperty(block=wafv2.CfnWebACL.BlockActionProperty()),
            statement=wafv2.CfnWebACL.StatementProperty(
                rate_based_statement=wafv2.CfnWebACL.RateBasedStatementProperty(
                    aggregate_key_type="IP",
                    limit=rate_value,
                )
            ),
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True,
                metric_name=f"Custom-RateLimit{rate_value}",
                sampled_requests_enabled=True,
            ),
        )

    @staticmethod
    def __aws_ip_reputation_list():
        """Creates a WAF rule to block requests from known malicious IPs.

        It creates a rule using the
        AWSManagedRulesAmazonIpReputationList managed rule set.

        This blocks requests from known malicious IP addresses.

        It runs with the highest priority to block bad IPs early.

        It enables CloudWatch metrics and sampled request logging.

        Returns the rule property to add to the WAF WebACL.
        """

        return wafv2.CfnWebACL.RuleProperty(
            name="AWS-AWSManagedRulesAmazonIpReputationList",
            priority=0,
            override_action=wafv2.CfnWebACL.OverrideActionProperty(none={}),
            statement=wafv2.CfnWebACL.StatementProperty(
                managed_rule_group_statement=wafv2.CfnWebACL.ManagedRuleGroupStatementProperty(
                    name="AWSManagedRulesAmazonIpReputationList",
                    vendor_name="AWS",
                )
            ),
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True,
                metric_name="AWS-AWSManagedRulesAmazonIpReputationList",
                sampled_requests_enabled=True,
            ),
        )

    @staticmethod
    def __aws_common_rule(aws_common_excluded_rules):
        """Creates a WAF rule using the AWSManagedRulesCommonRuleSet.

        Parameters:

        - aws_common_excluded_rules: List of rule names to exclude from the common rule set.

        It creates a rule using the AWSManagedRulesCommonRuleSet managed rule group.

        This enables a broad set of common protections like XSS, protocol violations, etc.

        It excludes any rules specified in the aws_common_excluded_rules list.

        This allows customizing the enabled protections.

        It enables CloudWatch metrics and sampled request logging.

        Returns the rule property to add to the WAF WebACL.
        """

        return wafv2.CfnWebACL.RuleProperty(
            name="AWS-AWSManagedRulesCommonRuleSet",
            priority=2,
            override_action=wafv2.CfnWebACL.OverrideActionProperty(none={}),
            statement=wafv2.CfnWebACL.StatementProperty(
                managed_rule_group_statement=wafv2.CfnWebACL.ManagedRuleGroupStatementProperty(
                    excluded_rules=aws_common_excluded_rules,
                    name="AWSManagedRulesCommonRuleSet",
                    vendor_name="AWS",
                )
            ),
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True,
                metric_name="AWS-AWSManagedRulesCommonRuleSet",
                sampled_requests_enabled=True,
            ),
        )

    @staticmethod
    def __aws_anonymous_list():
        """Creates a WAF rule to block requests from anonymous proxy servers.

        It creates a rule using the AWSManagedRulesAnonymousIpList
        managed rule set.

        This blocks requests from known anonymous proxy servers.

        It enables CloudWatch metrics and sampled request logging.

        Returns the rule property to add to the WAF WebACL.
        """

        return wafv2.CfnWebACL.RuleProperty(
            name="AWS-AWSManagedRulesAnonymousIpList",
            priority=3,
            override_action=wafv2.CfnWebACL.OverrideActionProperty(none={}),
            statement=wafv2.CfnWebACL.StatementProperty(
                managed_rule_group_statement=wafv2.CfnWebACL.ManagedRuleGroupStatementProperty(
                    name="AWSManagedRulesAnonymousIpList",
                    vendor_name="AWS",
                )
            ),
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True,
                metric_name="AWS-AWSManagedRulesAnonymousIpList",
                sampled_requests_enabled=True,
            ),
        )

    def web_acl(
        self,
        name: str,
        rate_value: int | None,
        aws_common_rule: bool = True,  # noqa: FBT001, FBT002
        aws_common_rule_ignore_list: list | None = None,
        aws_anony_list: bool = False,  # noqa: FBT001, FBT002
        aws_bad_inputs_rule: bool = False,  # noqa: FBT001, FBT002
        aws_sqli_rule: bool = False,  # noqa: FBT001, FBT002
        aws_account_takeover_prevention: bool | dict[Any, Any] = False,  # noqa: FBT001, FBT002
        waf_scope: Literal["REGIONAL", "CLOUDFRONT"] = "REGIONAL",
    ) -> wafv2.CfnWebACL:
        """Creates a WAF WebACL with configured rules.

        Parameters:

        - name: Name of the WebACL.
        - rate_value: Rate limit per IP if enabled.
        - aws_common_rule: Whether to enable AWS common rules.
        - aws_common_rule_ignore_list: List of common rules to exclude.
        - aws_anony_list: Whether to enable anonymous IP blocking.
        - aws_bad_inputs_rule: Whether to enable bad inputs rule.
        - aws_sqli_rule: Whether to enable SQLi rule.
        - aws_account_takeover_prevention: Config for account takeover prevention.
        - waf_scope: WAF scope - regional or Cloudfront.

        It enables the AWS IP reputation list rule by default.

        It adds additional rules based on input params:
        - Rate limit
        - AWS common rules
        - Anonymous IP blocking
        - Bad inputs blocking
        - SQLi blocking
        - Account takeover prevention

        Returns the configured CfnWebACL.
        """

        # 0. Reputation List. The first rule is enabled by default
        aws_ip_rep_list = self.__aws_ip_reputation_list()
        waf_rules = [aws_ip_rep_list]

        if rate_value:
            # 1. Custom Rate Limit
            rate_list = self.__rate_list(rate_value)
            waf_rules.append(rate_list)

        # 2. Common Rule
        aws_common_excluded_rules = None
        if aws_common_rule_ignore_list:
            aws_common_excluded_rules = [
                wafv2.CfnWebACL.ExcludedRuleProperty(name=rule_name) for rule_name in aws_common_rule_ignore_list
            ]
        if aws_common_rule:
            aws_common_rule = self.__aws_common_rule(aws_common_excluded_rules)
            waf_rules.append(aws_common_rule)

        if aws_anony_list:
            # 3. AnonymousIpList
            aws_anony_list = self.__aws_anonymous_list()
            waf_rules.append(aws_anony_list)

        if aws_bad_inputs_rule:
            # 4. Known Bad Inputs Rule
            aws_bad_inputs_rule = self.__aws_bad_inputs_rule()
            waf_rules.append(aws_bad_inputs_rule)

        if aws_sqli_rule:
            # 5. SQLi Rule
            aws_sqli_rule = self.__aws_sqli_rule()
            waf_rules.append(aws_sqli_rule)

        if aws_account_takeover_prevention:
            # 6. Account takeover prevention
            aws_account_takeover_prevention_rule = self.__aws_account_takeover_prevention(
                aws_account_takeover_prevention
            )
            waf_rules.append(aws_account_takeover_prevention_rule)

        return wafv2.CfnWebACL(
            self,
            "WAF ACL",
            default_action=wafv2.CfnWebACL.DefaultActionProperty(allow={}),
            scope=waf_scope,
            visibility_config=wafv2.CfnWebACL.VisibilityConfigProperty(
                cloud_watch_metrics_enabled=True, metric_name="web-acl", sampled_requests_enabled=True
            ),
            name=name,
            rules=waf_rules,
        )

    def web_acl_association(self, resource_arn, web_acl_arn: str) -> wafv2.CfnWebACLAssociation:
        """Associates a WAF WebACL with a resource.

        Parameters:

        - resource_arn: ARN of the resource to associate the WebACL with.
        - web_acl_arn: ARN of the WebACL to associate.

        Creates an association between the WebACL and the resource.

        This applies the WebACL rules to the resource.

        Returns the CfnWebACLAssociation.
        """

        return wafv2.CfnWebACLAssociation(self, "ACLAssociation", resource_arn=resource_arn, web_acl_arn=web_acl_arn)

    def web_acl_log(self, web_acl_arn: str, log_group_name: str) -> wafv2.CfnLoggingConfiguration:
        """Creates a WAF logging configuration to send logs to CloudWatch Logs.

        Parameters:

        - web_acl_arn: ARN of the WAF WebACL to enable logging for.
        - log_group_name: Name of the CloudWatch Logs group to send logs to.

        It creates a CloudWatch Logs group with 1-week retention.

        It configures the WAF WebACL to send logs to the Logs group.

        Returns the CfnLoggingConfiguration to enable logging.
        """

        log_group = logs.LogGroup(
            self,
            id="web_acl_log_group",
            log_group_name=log_group_name,
            retention=logs.RetentionDays.ONE_WEEK,
            removal_policy=cdk.RemovalPolicy.DESTROY,
        )

        return wafv2.CfnLoggingConfiguration(
            self,
            id="web_acl_cfn_log_configuration",
            log_destination_configs=[
                cdk.Stack.of(self).format_arn(
                    arn_format=cdk.ArnFormat.COLON_RESOURCE_NAME,
                    service="logs",
                    resource="log-group",
                    resource_name=log_group.log_group_name,
                )
            ],
            resource_arn=web_acl_arn,
        )



"""Validate variables against pydantic models."""

from typing import Literal

from pydantic import BaseModel, EmailStr, PositiveFloat


class Observability(BaseModel):
    """Defines the Observability model.

    Parameters:
      - None

    Attributes:

      - LOG_LEVEL (Literal["DEBUG", "INFO", "ERROR", "CRITICAL", "WARNING", "EXCEPTION"]):
        The log level to use.

      - LOG_SAMPLING_RATE (PositiveFloat):
        The log sampling rate.

    Functionality:

      - Defines a Pydantic model for observability configuration.
      - Constrains LOG_LEVEL to a set of literal string values.
      - Constrains LOG_SAMPLING_RATE to be a positive float.
    """

    LOG_LEVEL: Literal["DEBUG", "INFO", "ERROR", "CRITICAL", "WARNING", "EXCEPTION"]
    LOG_SAMPLING_RATE: PositiveFloat


class PipelinePluginsVars(BaseModel):
    """Defines the PipelinePluginsVars model.

    Parameters:
      - None

    Attributes:

      - pipeline_trigger (bool | None): Whether to enable the pipeline trigger plugin.
      optional.

      - pipeline_trigger_ssm_parameters (list | None): List of SSM parameters to trigger the pipeline.
      optional.

    Functionality:

      - defines a Pydantic model for pipeline plugins configuration.
      - pipeline_trigger enables/disables the pipeline trigger plugin.
      - pipeline_trigger_ssm_parameters configures SSM parameters that will trigger the pipeline when changed.
      - all attributes are optional.
    """

    pipeline_trigger: bool | None = None
    pipeline_trigger_ssm_parameters: list | None = None


class ConfigurationVars(PipelinePluginsVars):
    """Defines the ConfigurationVars model.

    Parameters:
      - None

    Attributes:

      - stage (Literal["dev", "ppe", "prod", "dr"]): The deployment stage.
      constrained to "dev", "ppe", "prod" or "dr".

      - alarm_emails (list[EmailStr]): A list of emails to receive alarm notifications.

      - project (str): The project name.

    Functionality:

      - extends the PipelineVars model with additional configuration attributes.
      - Constrains stage to a predefined set of options.
      - Allows configuring multiple alarm notification emails.
    """

    project: str
    stage: Literal["dev", "ppe", "prod", "dr"]
    alarm_emails: list[EmailStr]
    plugins: PipelinePluginsVars


class NotificationVars(BaseModel):
    """Defines the NotificationVars model.

    Parameters:
      - None

    Attributes:

      - slack_channel_id_alarms (constr | None): Optional Slack channel for alarm notifications.
      constrained to 11 characters.

      - slack_workspace_id (constr): Required Slack workspace ID. Constrained to 11 characters.

    Functionality:

      - defines a Pydantic model for notification configuration.
      - Allows configuring different Slack channels for alarms and general notifications.
      - slack_workspace_id is required.
      other attributes are optional.
    """

    slack_channel_id_alarms: str
    slack_workspace_id: str | None = None
